def spectrogram(waveform: Tensor,
    pad: int,
    window: Tensor,
    n_fft: int,
    hop_length: int,
    win_length: int,
    power: Optional[float],
    normalized: Union[str, bool],
    center: bool=True,
    pad_mode: str="reflect",
    onesided: bool=True,
    return_complex: Optional[bool]=None) -> Tensor:
  _0 = "`return_complex` argument is now deprecated and is not effective.`torchaudio.functional.spectrogram(power=None)` always returns a tensor with complex dtype. Please remove the argument in the function call."
  _1 = __torch__.torchaudio.functional.functional._get_spec_norms
  if torch.__isnot__(return_complex, None):
    torch.warn(_0)
  else:
    pass
  if torch.gt(pad, 0):
    waveform0 = torch.pad(waveform, [pad, pad])
  else:
    waveform0 = waveform
  frame_length_norm, window_norm, = _1(normalized, )
  shape = torch.size(waveform0)
  waveform1 = torch.reshape(waveform0, [-1, shape[-1]])
  spec_f = __torch__.torch.functional.stft(waveform1, n_fft, hop_length, win_length, window, center, pad_mode, frame_length_norm, onesided, True, )
  _2 = torch.add(torch.slice(shape, None, -1), torch.slice(torch.size(spec_f), -2))
  spec_f0 = torch.reshape(spec_f, _2)
  if window_norm:
    _3 = torch.sqrt(torch.sum(torch.pow(window, 2.)))
    spec_f1 = torch.div_(spec_f0, _3)
  else:
    spec_f1 = spec_f0
  if torch.__isnot__(power, None):
    power0 = unchecked_cast(float, power)
    if torch.eq(power0, 1.):
      _5 = torch.abs(spec_f1)
    else:
      _6 = torch.pow(torch.abs(spec_f1), power0)
      _5 = _6
    _4 = _5
  else:
    _4 = spec_f1
  return _4
def mask_along_axis(specgram: Tensor,
    mask_param: int,
    mask_value: float,
    axis: int,
    p: float=1.) -> Tensor:
  _7 = "Spectrogram must have at least two dimensions (time and frequency) ({} given)."
  _8 = "Only Frequency and Time masking are supported (axis {} and axis {} supported; {} given)."
  _9 = "The value of p must be between 0.0 and 1.0 ({} given)."
  _10 = __torch__.torchaudio.functional.functional._get_mask_param
  _11 = "Number of columns to be masked should be less than mask_param"
  dim = torch.dim(specgram)
  if torch.lt(dim, 2):
    ops.prim.RaiseException(torch.format(_7, dim), "builtins.ValueError")
  else:
    pass
  _12 = [torch.sub(dim, 2), torch.sub(dim, 1)]
  _13 = torch.__not__(torch.__contains__(_12, axis))
  if _13:
    _14 = torch.format(_8, torch.sub(dim, 2), torch.sub(dim, 1), axis)
    ops.prim.RaiseException(_14, "builtins.ValueError")
  else:
    pass
  if torch.le(0., p):
    _15 = torch.le(p, 1.)
  else:
    _15 = False
  if torch.__not__(_15):
    ops.prim.RaiseException(torch.format(_9, p), "builtins.ValueError")
  else:
    pass
  mask_param0 = _10(mask_param, p, (torch.size(specgram))[axis], )
  if torch.lt(mask_param0, 1):
    _16 = specgram
  else:
    shape = torch.size(specgram)
    _17 = torch.add([-1], torch.list(torch.slice(shape, -2)))
    specgram0 = torch.reshape(specgram, _17)
    value = torch.mul(torch.rand([1]), mask_param0)
    _18 = torch.rand([1])
    _19 = torch.size(specgram0, torch.add(torch.sub(axis, dim), 3))
    min_value = torch.mul(_18, torch.add(torch.neg(value), _19))
    mask_start = torch.squeeze(torch.to(min_value, 4))
    _20 = torch.add(torch.to(min_value, 4), torch.to(value, 4))
    mask_end = torch.squeeze(_20)
    _21 = (torch.size(specgram0))[torch.add(torch.sub(axis, dim), 3)]
    _22 = ops.prim.device(specgram0)
    mask = torch.arange(0, _21, dtype=ops.prim.dtype(specgram0), layout=None, device=_22)
    mask0 = torch.__and__(torch.ge(mask, mask_start), torch.lt(mask, mask_end))
    if torch.eq(axis, torch.sub(dim, 2)):
      mask1 = torch.unsqueeze(mask0, -1)
    else:
      mask1 = mask0
    _23 = torch.ge(torch.sub(mask_end, mask_start), mask_param0)
    if bool(_23):
      ops.prim.RaiseException(_11, "builtins.ValueError")
    else:
      pass
    specgram1 = torch.masked_fill(specgram0, mask1, mask_value)
    _24 = torch.slice(shape, None, -2)
    _25 = torch.slice(torch.size(specgram1), -2)
    specgram2 = torch.reshape(specgram1, torch.add(_24, _25))
    _16 = specgram2
  return _16
def _get_spec_norms(normalized: Union[str, bool]) -> Tuple[bool, bool]:
  _26 = isinstance(normalized, str)
  if _26:
    normalized0 = unchecked_cast(str, normalized)
    _27 = torch.__contains__(["frame_length", "window"], normalized0)
    if torch.__not__(_27):
      _28 = torch.format("Invalid normalized parameter: {}", normalized0)
      ops.prim.RaiseException(_28, "builtins.ValueError")
    else:
      pass
    _29 = torch.eq(normalized0, "frame_length")
    if _29:
      frame_length_norm0, window_norm0 = True, False
    else:
      if torch.eq(normalized0, "window"):
        window_norm1 = True
      else:
        window_norm1 = False
      frame_length_norm0, window_norm0 = False, window_norm1
    frame_length_norm, window_norm = frame_length_norm0, window_norm0
  else:
    normalized1 = unchecked_cast(bool, normalized)
    normalized2 = unchecked_cast(bool, normalized1)
    if normalized2:
      window_norm2 = True
    else:
      window_norm2 = False
    frame_length_norm, window_norm = False, window_norm2
  return (frame_length_norm, window_norm)
def _get_mask_param(mask_param: int,
    p: float,
    axis_length: int) -> int:
  if torch.eq(p, 1.):
    _30 = mask_param
  else:
    _31 = ops.prim.min(mask_param, int(torch.mul(axis_length, p)))
    _30 = _31
  return _30
